pub struct LexerConfig {
pub use_origin_string: bool,
pub reserve_comments: bool,
}
122
123
124
125
126
127
128
pub fn new() -> Self {
Lexer {
debug: false,
tokens: Vec::<Token>::new(),
config: LexerConfig::default(),
}
}
Immutably borrows from an owned value.
Read more
Mutably borrows from an owned value.
Read more
Returns the argument unchanged.
Calls U::from(self).
That is, this conversion is whatever the implementation of
From<T> for U chooses to do.
The type returned in the event of a conversion error.
Performs the conversion.
The type returned in the event of a conversion error.
Performs the conversion.